import { Ref, ComputedRef, VNodeProps, defineComponent, UnwrapRef } from 'vue';

/**
 * Value may or may not be a `ref`.
 *
 * ```ts
 * type MaybeRef<T> = T | Ref<T>
 * ```
 */
type MaybeRef<T> = T | Ref<T> | ComputedRef<T>;
/**
 * May be a simple ref (nor computed ref)
 */
type MaybeBaseRef<T> = T | Ref<T>;
/** VNodeProps Object */
interface MergedVNodeProps extends VNodeProps {
    ref: TemplateRef | ((el: TemplateRef | null) => void);
}
/**
 * Any function
 */
type Fn = () => void;

/**
 * Interface for node provided by template ref
 */
type TemplateRef = Element | VueComponentInstance | undefined | null;
/**
 * For internal use
 *
 * Creates refs that will be bound to the template/render function.
 *
 * Why not just use the regular `ref(null)` and bind it to the element?
 *
 * 1. To avoid unwrapping template refs which maybe components. This hook will always
 *    give us the actual element being bound the the element, and not the component
 *    options.
 *
 * 2. In some cases where we need an up-to-date value of the ref node,
 *    from the consuming component, we can use this hook.
 *
 * @returns []
 */
declare function useRef<T extends HTMLElement>(): [
    (el: TemplateRef | null) => void,
    Ref<T | null>
];
/** Vue Component HTML Element Instance */
type VueComponentInstance = InstanceType<ReturnType<typeof defineComponent>>;
/** Ref may or may not be an HTML Element or VueComponent instance */
type MaybeElementRef = MaybeRef<Element | VueComponentInstance | undefined | null>;
/**
 * Unwraps element from ref
 * @param elementRef Ref of template node
 */
declare function unrefElement(elementRef: MaybeElementRef): UnwrapRef<MaybeElementRef>;
/**
 * Creates a ref whose value updates are debounced
 *
 * @example Simple example
 *
 * ```ts
 * const foo = useDebouncedRef('bar')
 * foo.value = 'baz'
 *
 * // foo.value to be updated to 'baz' after the delay of 300ms
 * ```
 *
 * @example Custom delay
 *
 * ```ts
 * const foo = useDebouncedRef('bar', 500)
 * foo.value = 'baz'
 *
 * // foo.value to be updated to 'baz' after the delay of 500ms
 * ```
 */
declare function useDebouncedRef<T = unknown>(initialValue: T, delay?: number, immediate?: boolean): Ref<UnwrapRef<T>>;
type DebouncedRef = Ref<unknown>;
declare function contains(containers: Set<HTMLElement>, element: HTMLElement): boolean;

export { DebouncedRef as D, Fn as F, MaybeElementRef as M, TemplateRef as T, VueComponentInstance as V, unrefElement as a, useDebouncedRef as b, contains as c, MaybeRef as d, MaybeBaseRef as e, MergedVNodeProps as f, useRef as u };
